@@ -214,6 +214,7 @@ static int stress_dup(stress_args_t *args)
 		PROT_READ | PROT_WRITE,
 		MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (info != MAP_FAILED) {
+		stress_set_vma_anon_name(info, sizeof(*info), "dup-race-context");
 		if (stress_temp_dir_mk(args->name, args->pid, args->instance) < 0) {
 			rc = EXIT_NO_RESOURCE;
 			goto tidy_mmap;
@@ -234,6 +235,8 @@ static int stress_dup(stress_args_t *args)
 		goto tidy_fds;
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -242,6 +245,11 @@ static int stress_dup(stress_args_t *args)
 		for (n = 1; n < max_fd; n++) {
 			int tmp;
 			double t;
+#if defined(O_CLOEXEC)
+			const int flags = O_CLOEXEC;
+#else
+			const int flags = 0;
+#endif
 
 			t = stress_time_now();
 			fds[n] = dup(fds[0]);
@@ -259,7 +267,7 @@ static int stress_dup(stress_args_t *args)
 				break;
 
 			/* do an invalid dup3 on an invalid fd */
-			tmp = shim_dup3(fds[0], bad_fd, O_CLOEXEC);
+			tmp = shim_dup3(fds[0], bad_fd, flags);
 			if (UNLIKELY(tmp >= 0))
 				(void)close(tmp);
 			else if (errno == ENOSYS)
@@ -289,7 +297,7 @@ static int stress_dup(stress_args_t *args)
 				break;
 
 			/* do an invalid dup3 with same oldfd and newfd */
-			tmp = shim_dup3(fds[0], fds[0], O_CLOEXEC);
+			tmp = shim_dup3(fds[0], fds[0], flags);
 			if (UNLIKELY(tmp >= 0))
 				(void)close(tmp);
 			else if (errno == ENOSYS)
@@ -302,7 +310,7 @@ static int stress_dup(stress_args_t *args)
 				int fd;
 
 				t = stress_time_now();
-				fd = shim_dup3(fds[0], fds[n], O_CLOEXEC);
+				fd = shim_dup3(fds[0], fds[n], flags);
 				/* No dup3 support? then fallback to dup2 */
 				if ((fd < 0) && (errno == ENOSYS)) {
 					t = stress_time_now();
@@ -326,24 +334,29 @@ static int stress_dup(stress_args_t *args)
 			if (!stress_continue(args))
 				break;
 
-			t = stress_time_now();
-			fds[n] = dup2(fds[0], fds[n]);
-			if (LIKELY(fds[n] >= 0)) {
-				dup_duration += stress_time_now() - t;
-				dup_count += 1;
-			} else {
-				break;
+			if (fds[n] > -1) {
+				t = stress_time_now();
+				fds[n] = dup2(fds[0], fds[n]);
+				if (LIKELY(fds[n] >= 0)) {
+					dup_duration += stress_time_now() - t;
+					dup_count += 1;
+				} else {
+					break;
+				}
 			}
 
 			if (!stress_continue(args))
 				break;
 
 			/* dup2 on the same fd should be a no-op */
-			tmp = dup2(fds[n], fds[n]);
-			if (UNLIKELY(tmp != fds[n])) {
-				pr_fail("%s: dup2 failed with same fds, errno=%d (%s)\n",
-					args->name, errno, strerror(errno));
-				break;
+			if (fds[n] > -1) {
+				tmp = dup2(fds[n], fds[n]);
+				if (UNLIKELY(tmp != fds[n])) {
+					pr_fail("%s: dup2 failed with same fds, errno=%d (%s)\n",
+						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+					break;
+				}
 			}
 			/* do an invalid dup2 on an invalid fd */
 			tmp = dup2(fds[0], bad_fd);
@@ -403,12 +416,14 @@ tidy_mmap:
 #endif
 	rate = (dup_count > 0.0) ? (double)dup_duration / dup_count : 0.0;
 	stress_metrics_set(args, 0, "nanosecs per dup call",
-		rate * STRESS_DBL_NANOSECOND, STRESS_HARMONIC_MEAN);
+		rate * STRESS_DBL_NANOSECOND, STRESS_METRIC_HARMONIC_MEAN);
+	stress_metrics_set(args, 0, "dup calls",
+		dup_count, STRESS_METRIC_TOTAL);
 
 	return rc;
 }
 
-stressor_info_t stress_dup_info = {
+const stressor_info_t stress_dup_info = {
 	.stressor = stress_dup,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
