@@ -3909,7 +3909,7 @@ static void NORETURN MLOCKED_TEXT stress_sig_handler(int signum)
 static inline int stress_do_syscall(
 	stress_args_t *args,
 	const long number,
-	const bool random)
+	const bool do_random)
 {
 	pid_t pid;
 	int rc = 0;
@@ -3993,7 +3993,7 @@ static inline int stress_do_syscall(
 
 		/* Add to known syscalls that don't return ENOSYS */
 		if (rc != ENOSYS) {
-			if ((!random) || (number < 65536))
+			if ((!do_random) || (number < 65536))
 				syscall_add(number);
 		}
 		stress_bogo_inc(args);
@@ -4012,8 +4012,9 @@ static int stress_enosys(stress_args_t *args)
 #if defined(STRESS_EXERCISE_X86_SYSCALL)
 	stress_x86syscall_available = stress_cpu_x86_has_syscall();
 #endif
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
 	if (!stress_continue(args))
 		return EXIT_SUCCESS;
@@ -4167,13 +4168,13 @@ deinit:
 
 	return EXIT_SUCCESS;
 }
-stressor_info_t stress_enosys_info = {
+const stressor_info_t stress_enosys_info = {
 	.stressor = stress_enosys,
 	.class = CLASS_OS,
 	.help = help
 };
 #else
-stressor_info_t stress_enosys_info = {
+const stressor_info_t stress_enosys_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_OS,
 	.help = help,
