@@ -34,6 +34,7 @@ static const stress_help_t help[] = {
  */
 static int stress_io(stress_args_t *args)
 {
+	int rc = EXIT_SUCCESS;
 #if defined(HAVE_SYNCFS)
 	int i, fd, n_mnts;
 	char *mnts[MAX_MNTS];
@@ -49,14 +50,19 @@ static int stress_io(stress_args_t *args)
 
 	fd = openat(AT_FDCWD, ".", O_RDONLY | O_NONBLOCK | O_DIRECTORY);
 #endif
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		sync();
 #if defined(HAVE_SYNCFS)
-		if ((fd != -1) && (syncfs(fd) < 0))
+		if ((fd != -1) && (syncfs(fd) < 0)) {
 			pr_fail("%s: syncfs failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
+			rc = EXIT_FAILURE;
+			goto tidy;
+		}
 
 		/* try to sync on all the mount points */
 		for (i = 0; i < n_mnts; i++) {
@@ -68,6 +74,8 @@ static int stress_io(stress_args_t *args)
 				    (errno != EINTR)) {
 					pr_fail("%s: syncfs failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+					goto tidy;
 				}
 			}
 		}
@@ -78,6 +86,8 @@ static int stress_io(stress_args_t *args)
 		if (syncfs(bad_fd) == 0) {
 			pr_fail("%s: syncfs on invalid fd %d succeed\n",
 				args->name, bad_fd);
+			rc = EXIT_FAILURE;
+			goto tidy;
 		}
 #else
 		UNEXPECTED
@@ -85,6 +95,9 @@ static int stress_io(stress_args_t *args)
 		stress_bogo_inc(args);
 	} while (stress_continue(args));
 
+#if defined(HAVE_SYNCFS)
+tidy:
+#endif
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 #if defined(HAVE_SYNCFS)
 	if (fd != -1)
@@ -97,10 +110,10 @@ static int stress_io(stress_args_t *args)
 	UNEXPECTED
 #endif
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
-stressor_info_t stress_io_info = {
+const stressor_info_t stress_io_info = {
 	.stressor = stress_io,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
