@@ -718,30 +718,6 @@ static void stress_memthrash_random(const stress_memthrash_context_t *context, s
 	}
 }
 
-/*
- *  stress_set_memthrash_method()
- *	set the default memthresh method
- */
-static int stress_set_memthrash_method(const char *name)
-{
-	size_t i;
-
-	for (i = 0; i < SIZEOF_ARRAY(memthrash_methods); i++) {
-		if (!strcmp(memthrash_methods[i].name, name)) {
-			stress_set_setting("memthrash-method", TYPE_ID_SIZE_T, &i);
-			return 0;
-		}
-	}
-
-	(void)fprintf(stderr, "memthrash-method must be one of:");
-	for (i = 0; i < SIZEOF_ARRAY(memthrash_methods); i++) {
-		(void)fprintf(stderr, " %s", memthrash_methods[i].name);
-	}
-	(void)fprintf(stderr, "\n");
-
-	return -1;
-}
-
 static void stress_memthrash_find_primes(void)
 {
 	size_t i;
@@ -946,7 +922,7 @@ static int stress_memthrash(stress_args_t *args)
 			if (!context.numa_node_mask) {
 				if (args->instance == 0) {
 					pr_inf_skip("%s: could not allocate %zd numa elements in numa mask, ignoring numa memthrash stressor\n",
-						args->name, numa_elements);	
+						args->name, numa_elements);
 				}
 				context.numa_node_mask = NULL;
 				context.numa_node_mask_size = 0;
@@ -975,8 +951,9 @@ static int stress_memthrash(stress_args_t *args)
 
 	(void)sigfillset(&set);
 
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	rc = stress_oomable_child(args, &context, stress_memthrash_child, STRESS_OOMABLE_NORMAL);
 
@@ -988,36 +965,33 @@ static int stress_memthrash(stress_args_t *args)
 	return rc;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_memthrash_method,	stress_set_memthrash_method },
-	{ 0,			NULL }
+static const char *stress_memthrash_method(const size_t i)
+{
+	return (i < SIZEOF_ARRAY(memthrash_methods)) ? memthrash_methods[i].name : NULL;
+}
+
+static const stress_opt_t opts[] = {
+	{ OPT_memthrash_method, "memthrash-method", TYPE_ID_SIZE_T_METHOD, 0, 0, stress_memthrash_method },
+	END_OPT,
 };
 
-stressor_info_t stress_memthrash_info = {
+const stressor_info_t stress_memthrash_info = {
 	.stressor = stress_memthrash,
 	.class = CLASS_MEMORY,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.help = help
 };
 #else
 
-static int stress_set_memthrash_method(const char *name)
-{
-	(void)name;
-
-	(void)pr_inf("warning: --memthrash-method not available on this system\n");
-	return 0;
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_memthrash_method,	stress_set_memthrash_method },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_memthrash_method, "memthrash-method", TYPE_ID_SIZE_T_METHOD, 0, 0, stress_unimplemented_method },
+	END_OPT,
 };
 
-stressor_info_t stress_memthrash_info = {
+const stressor_info_t stress_memthrash_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_MEMORY,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.help = help,
 	.unimplemented_reason = "built without pthread support"
 };
