@@ -19,6 +19,7 @@
  */
 #include "stress-ng.h"
 #include "core-sort.h"
+#include "core-builtin.h"
 #include "core-target-clones.h"
 
 #define MIN_MERGESORT_SIZE	(1 * KB)
@@ -45,11 +46,11 @@ typedef struct {
 
 #define IDX(base, idx, size)	((base) + ((idx) * (size)))
 
-static inline ALWAYS_INLINE void mergesort_copy(uint8_t *RESTRICT p1, uint8_t *RESTRICT p2, register size_t size)
+static inline void ALWAYS_INLINE mergesort_copy(uint8_t *RESTRICT p1, uint8_t *RESTRICT p2, register size_t size)
 {
+	register const uint32_t *u32end = (uint32_t *)(p1 + size);
 	register uint32_t *u32p1 = (uint32_t *)p1;
 	register uint32_t *u32p2 = (uint32_t *)p2;
-	register const uint32_t *u32end = (uint32_t *)(p1 + size);
 
 	while (LIKELY(u32p1 < u32end))
 		*(u32p1++) = *(u32p2++);
@@ -77,12 +78,10 @@ static inline void mergesort_partition4(
 		mergesort_partition4(base, lhs, mid + 1, right, compar);
 
 	lhs_len = mid - left + 1;
-	rhs_len = right - mid;
-
 	lhs_size = lhs_len * 4;
-	rhs_size = rhs_len * 4;
-
 	rhs = lhs + lhs_size;
+	rhs_len = right - mid;
+	rhs_size = rhs_len * 4;
 
 	mergesort_copy(lhs, IDX(base, left, 4), lhs_size);
 	mergesort_copy(rhs, IDX(base, (mid + 1), 4), rhs_size);
@@ -137,12 +136,10 @@ static inline void mergesort_partition(
 		mergesort_partition(base, lhs, mid + 1, right, size, compar);
 
 	lhs_len = mid - left + 1;
-	rhs_len = right - mid;
-
 	lhs_size = lhs_len * size;
-	rhs_size = rhs_len * size;
-
 	rhs = lhs + lhs_size;
+	rhs_len = right - mid;
+	rhs_size = rhs_len * size;
 
 	mergesort_copy(lhs, IDX(base, left, size), lhs_size);
 	mergesort_copy(rhs, IDX(base, (mid + 1), size), rhs_size);
@@ -153,13 +150,13 @@ static inline void mergesort_partition(
 
 	while ((lhs < lhs_end) && (rhs < rhs_end)) {
 		if (compar(lhs, rhs) < 0) {
-			mergesort_copy(base, lhs, size);
+			shim_memcpy(base, lhs, size);
 			lhs += size;
 			if (lhs > lhs_end)
 				break;
 			base += size;
 		} else {
-			mergesort_copy(base, rhs, size);
+			shim_memcpy(base, rhs, size);
 			rhs += size;
 			if (rhs > rhs_end)
 				break;
@@ -212,43 +209,15 @@ static const stress_mergesort_method_t stress_mergesort_methods[] = {
 	{ "mergesort-nonlibc",	mergesort_nonlibc },
 };
 
-static int stress_set_mergesort_method(const char *opt)
+static const char *stress_mergesort_method(const size_t i)
 {
-	size_t i;
-
-	for (i = 0; i < SIZEOF_ARRAY(stress_mergesort_methods); i++) {
-		if (strcmp(opt, stress_mergesort_methods[i].name) == 0) {
-			stress_set_setting("mergesort-method", TYPE_ID_SIZE_T, &i);
-			return 0;
-		}
-	}
-
-	(void)fprintf(stderr, "mergesort-method must be one of:");
-	for (i = 0; i < SIZEOF_ARRAY(stress_mergesort_methods); i++) {
-		(void)fprintf(stderr, " %s", stress_mergesort_methods[i].name);
-	}
-	(void)fprintf(stderr, "\n");
-	return -1;
+	return (i < SIZEOF_ARRAY(stress_mergesort_methods)) ? stress_mergesort_methods[i].name : NULL;
 }
 
-/*
- *  stress_set_mergesort_size()
- *	set mergesort size
- */
-static int stress_set_mergesort_size(const char *opt)
-{
-	uint64_t mergesort_size;
-
-	mergesort_size = stress_get_uint64(opt);
-	stress_check_range("mergesort-size", mergesort_size,
-		MIN_MERGESORT_SIZE, MAX_MERGESORT_SIZE);
-	return stress_set_setting("mergesort-size", TYPE_ID_UINT64, &mergesort_size);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_mergesort_size,	stress_set_mergesort_size },
-	{ OPT_mergesort_method,	stress_set_mergesort_method },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_mergesort_size,   "mergesort-size",   TYPE_ID_UINT64, MIN_MERGESORT_SIZE, MAX_MERGESORT_SIZE, NULL },
+	{ OPT_mergesort_method, "mergesort-method", TYPE_ID_SIZE_T_METHOD, 0, 0, stress_mergesort_method },
+	END_OPT,
 };
 
 #if !defined(__OpenBSD__) &&	\
@@ -328,8 +297,10 @@ static int stress_mergesort(stress_args_t *args)
 #endif
 
 	stress_sort_data_int32_init(data, n);
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		double t;
@@ -443,10 +414,10 @@ tidy:
 	return rc;
 }
 
-stressor_info_t stress_mergesort_info = {
+const stressor_info_t stress_mergesort_info = {
 	.stressor = stress_mergesort,
-	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY | CLASS_SEARCH,
-	.opt_set_funcs = opt_set_funcs,
+	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY | CLASS_SORT,
+	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
