@@ -44,11 +44,6 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,			NULL }
 };
 
-static int stress_set_pipeherd_yield(const char *opt)
-{
-	return stress_set_setting_true("pipeherd-yield", opt);
-}
-
 static int stress_pipeherd_read_write(stress_args_t *args, const int fd[2], const bool pipeherd_yield)
 {
 	while (stress_continue(args)) {
@@ -132,8 +127,9 @@ static int stress_pipeherd(stress_args_t *args)
 	for (i = 0; i < PIPE_HERD_MAX; i++)
 		pids[i] = -1;
 
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 #if defined(HAVE_GETRUSAGE) &&	\
     defined(RUSAGE_CHILDREN) &&	\
@@ -215,15 +211,15 @@ static int stress_pipeherd(stress_args_t *args)
 	return EXIT_SUCCESS;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_pipeherd_yield,	stress_set_pipeherd_yield },
-	{ 0,                    NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_pipeherd_yield, "pipeherd-yield", TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
 };
 
-stressor_info_t stress_pipeherd_info = {
+const stressor_info_t stress_pipeherd_info = {
 	.stressor = stress_pipeherd,
-	.class = CLASS_PIPE_IO | CLASS_MEMORY | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.class = CLASS_PIPE_IO | CLASS_MEMORY | CLASS_OS | CLASS_IPC,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
