@@ -35,7 +35,6 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		   NULL }
 };
 
-
 #define	MIN_BLKSZ	((int)512)
 #define	MAX_BLKSZ	((int)(128 * KB))
 
@@ -347,48 +346,25 @@ static int stress_rawdev_all(
 	return ret;
 }
 
-/*
- *  stress_set_rawdev_method()
- *	set the default rawdev method
- */
-static int stress_set_rawdev_method(const char *name)
+static const char *stress_rawdev_method(const size_t i)
 {
-	size_t i;
-
-	for (i = 0; i < SIZEOF_ARRAY(rawdev_methods); i++) {
-		if (!strcmp(rawdev_methods[i].name, name)) {
-			stress_set_setting("rawdev-method", TYPE_ID_SIZE_T, &i);
-			return 0;
-		}
-	}
-
-	(void)fprintf(stderr, "rawdev-method must be one of:");
-	for (i = 0; i < SIZEOF_ARRAY(rawdev_methods); i++) {
-		(void)fprintf(stderr, " %s", rawdev_methods[i].name);
-	}
-	(void)fprintf(stderr, "\n");
-
-	return -1;
+	return (i < SIZEOF_ARRAY(rawdev_methods)) ? rawdev_methods[i].name : NULL;
 }
-#else
-/*
- *  stress_set_rawdev_method()
- *	set the default rawdev method
- */
-static int stress_set_rawdev_method(const char *name)
-{
-	(void)name;
 
-	(void)fprintf(stderr, "option --rawdev-method not supported\n");
-	return -1;
-}
-#endif
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_rawdev_method,	stress_set_rawdev_method },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_rawdev_method, "rawdev-method", TYPE_ID_SIZE_T_METHOD, 0, 0, stress_rawdev_method },
+	END_OPT,
 };
 
+#else
+
+static const stress_opt_t opts[] = {
+	{ OPT_rawdev_method, "rawdev-method", TYPE_ID_SIZE_T_METHOD, 0, 0, stress_unimplemented_method },
+	END_OPT,
+};
+
+#endif
+
 #if defined(HAVE_SYS_SYSMACROS_H) &&	\
     defined(BLKGETSIZE) && 		\
     defined(BLKSSZGET)
@@ -473,6 +449,7 @@ static int stress_rawdev(stress_args_t *args)
 		free(metrics);
 		return EXIT_NO_RESOURCE;
 	}
+	stress_set_vma_anon_name(buffer, mmapsz, "io-buffer");
 
 	(void)close(fd);
 	fd = open(devpath, O_RDONLY | O_DIRECT);
@@ -488,8 +465,9 @@ static int stress_rawdev(stress_args_t *args)
 		pr_dbg("%s: exercising %s (%zd blocks of size %zd bytes)\n",
 			args->name, devpath, blks, blksz);
 
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		if (func(args, fd, buffer, blks, blksz, &metrics[rawdev_method]) < 0) {
@@ -521,19 +499,19 @@ static int stress_rawdev(stress_args_t *args)
 	return rc;
 }
 
-stressor_info_t stress_rawdev_info = {
+const stressor_info_t stress_rawdev_info = {
 	.stressor = stress_rawdev,
 	.supported = stress_rawdev_supported,
 	.class = CLASS_IO,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_rawdev_info = {
+const stressor_info_t stress_rawdev_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_IO,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without sys/sysmacros.h or undefined BLKGETSIZE, BLKSSZGET"
