@@ -105,8 +105,9 @@ static int stress_reboot(stress_args_t *args)
 	}
 #endif
 
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		int ret;
@@ -133,6 +134,11 @@ static int stress_reboot(stress_args_t *args)
 		ret = shim_reboot(0, 0, (int)SHIM_LINUX_REBOOT_CMD_RESTART, 0);
 		if (ret < 0) {
 			if (reboot_capable) {
+				if (errno == EPERM) {
+					pr_inf_skip("%s: no permission, skipping stressor\n", args->name);
+					rc = EXIT_NO_RESOURCE;
+					break;
+				}
 				if (errno != EINVAL) {
 					pr_fail("%s: reboot with incorrect magic didn't return EINVAL, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
@@ -150,6 +156,11 @@ static int stress_reboot(stress_args_t *args)
 		ret = shim_reboot(0, 0, (int)SHIM_LINUX_REBOOT_CMD_SW_SUSPEND, 0);
 		if (ret < 0) {
 			if (reboot_capable) {
+				if (errno == EPERM) {
+					pr_inf_skip("%s: no permission, skipping stressor\n", args->name);
+					rc = EXIT_NO_RESOURCE;
+					break;
+				}
 				if (errno != EINVAL) {
 					pr_fail("%s: reboot with incorrect magic didn't return EINVAL, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
@@ -193,7 +204,7 @@ static int stress_reboot(stress_args_t *args)
 	return EXIT_SUCCESS;
 }
 
-stressor_info_t stress_reboot_info = {
+const stressor_info_t stress_reboot_info = {
 	.stressor = stress_reboot,
 	.class = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
@@ -202,7 +213,7 @@ stressor_info_t stress_reboot_info = {
 
 #else
 
-stressor_info_t stress_reboot_info = {
+const stressor_info_t stress_reboot_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
